@@ -1,4 +1,4 @@
-/* Copyright (C) 2017-2020 Free Software Foundation, Inc.
+/* Copyright (C) 2017-2024 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -15,8 +15,8 @@
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
-#ifndef ARCH_I386_H
-#define ARCH_I386_H
+#ifndef GDB_ARCH_I386_H
+#define GDB_ARCH_I386_H
 
 #include "gdbsupport/tdesc.h"
 #include <stdint.h>
@@ -24,4 +24,4 @@
 target_desc *i386_create_target_description (uint64_t xcr0, bool is_linux,
 					     bool segments);
 
-#endif /* ARCH_I386_H */
+#endif /* GDB_ARCH_I386_H */
